val CPN'file = (ref NONE) : (TextIO.outstream option ref)

fun calculateTimeStamp() = 
let
    val curtimeint = IntInf.toLarge(time())*60;
    val curtime = SMLTime.fromSeconds(curtimeint);
    val curdate = Date.fromTimeLocal(curtime);
    val timestamp = (Date.fmt "%Y-%m-%dT%H:%M" curdate);

in
    timestamp^":"^"00.000+01:00"
end

fun writeTimeStamp(timestamp) = 
let
    val _ = TextIO.output(Option.valOf (!CPN'file), "<date key=\"time:timestamp\" value=\"")
    val _ = TextIO.output(Option.valOf (!CPN'file), timestamp)
in
    TextIO.output(Option.valOf (!CPN'file), "\"/>\n")
end

fun writeWorkflowModelElement(workflowModelElement) = 
let
    val _ = TextIO.output(Option.valOf (!CPN'file), "<string key=\"concept:name\" value=\"")
    val _ = TextIO.output(Option.valOf (!CPN'file), workflowModelElement)
in
    TextIO.output(Option.valOf (!CPN'file), "\"/>\n")
end

fun getDescription [] = ""
  | getDescription (hd::tl) = hd

fun getComplement(event, description) = 
let
    val desc = getDescription(description)
    val complement = "unknowntype=\"" ^ desc ^ "\""
in
    if event = "unknown"
    then  complement
    else  ""
end

fun writeEventType(event :: description) = 
let
    val complement = getComplement(event, description)
    val _ = TextIO.output(Option.valOf (!CPN'file), "<string key=\"lifecycle:transition\" value=\"")
    (*	val _ = TextIO.output(Option.valOf (!CPN'file), complement)
    val _ = TextIO.output(Option.valOf (!CPN'file), ">")		*)
    val _ = TextIO.output(Option.valOf (!CPN'file), event)
in
    TextIO.output(Option.valOf (!CPN'file), "\"/>\n")
end

fun writeOriginator(Originator) = 
let
    val _ = TextIO.output(Option.valOf (!CPN'file), "<string key=\"org:resource\" value=\"")
    val _ = TextIO.output(Option.valOf (!CPN'file), Originator)
in
    TextIO.output(Option.valOf (!CPN'file), "\"/>\n")
end

fun writeDataAttributes(nil) = ""
  | writeDataAttributes(name::nil) =  "<string key=\"unknown\" value=\"" ^ name ^ "\"/>\n" 
  | writeDataAttributes(name::value::tail) = "<string key=\"" ^ name ^ "\" value=\"" ^value ^"\"/>\n" ^ writeDataAttributes(tail) 

fun writeData(data) = 
    TextIO.output(Option.valOf (!CPN'file), writeDataAttributes(data))

fun writeCase(case_id) =
let
    val case_id' = Int.toString case_id
in
    TextIO.output(Option.valOf (!CPN'file), "<string key=\"cpntools:case\" value=\"" ^ case_id' ^ "\"/>\n")
end

fun add (caseID, workflowModelElement, EventType, TimeStamp, Originator, Data)=
let
    val _ = TextIO.output(Option.valOf (!CPN'file), "<trace>\n<event>\n")
    val _ = writeCase(caseID)
    val _ = writeData(Data)
    val _ = writeWorkflowModelElement(workflowModelElement)
    val _ = writeEventType(EventType)
    val _ = writeTimeStamp(TimeStamp)
    val _ = writeOriginator(Originator)
    val _ = TextIO.output(Option.valOf (!CPN'file), "</event>\n</trace>\n")
in
    ()
end

val addATE = add

fun createCaseFile(fileName) = 
let
    val file_id = TextIO.openOut(fileName)
    val _ = CPN'file := (SOME file_id)

    val _ = TextIO.output(Option.valOf (!CPN'file), "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n")
    val _ = TextIO.output(Option.valOf (!CPN'file), "<log xes.version=\"1.0\" xes.features=\"nested-attributes\" xmlns=\"http://www.xes-standard.org/\">\n")
in
    ()
end

fun closeCaseFile() = 
let
    val _ = TextIO.output(Option.valOf (!CPN'file), "</log>\n")
in
    TextIO.closeOut(Option.valOf(!CPN'file))
end;


